"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var vuex_1 = require("vuex");
var types_1 = require("./types");
var decorator_util_1 = require("./utils/decorator-util");
function createVuexStore(pModule, pOptions) {
    var storeProvider = {
        store: undefined
    };
    var userOptions = tslib_1.__assign({ modules: {}, plugins: [], strict: false }, pOptions);
    var options = transformRecursive(storeProvider, pModule);
    options.strict = userOptions.strict;
    options.modules = tslib_1.__assign({}, options.modules, userOptions.modules);
    options.plugins = userOptions.plugins;
    // TODO merge with options from params
    var store = new vuex_1.Store(options);
    store.__root__ = pModule;
    storeProvider.store = store;
    return store;
}
exports.createVuexStore = createVuexStore;
function useStore(vuexStore) {
    var typedStore = vuexStore;
    return typedStore.__root__;
}
exports.useStore = useStore;
function transformRecursive(pStoreProvider, pModule, pNamespace) {
    if (pNamespace === void 0) { pNamespace = []; }
    var storeOptions = {
        actions: {},
        getters: {},
        modules: {},
        mutations: {},
        namespaced: true,
        state: {}
    };
    var builder = {
        namespaces: pNamespace,
        options: storeOptions,
        provider: pStoreProvider
    };
    var constructor = pModule.constructor;
    var decorators = decorator_util_1.getDecorators(constructor);
    decorators.forEach(function (decorator) {
        var type = decorator.type, propertyName = decorator.propertyName;
        switch (type) {
            case types_1.DecoratorType.STATE:
                return bindState(builder, pModule, propertyName);
            case types_1.DecoratorType.GETTER:
                return bindGetter(builder, pModule, propertyName);
            case types_1.DecoratorType.MUTATION:
                return bindMutation(builder, pModule, propertyName);
            case types_1.DecoratorType.ACTION:
                return bindAction(builder, pModule, propertyName);
            case types_1.DecoratorType.MODULE:
                return bindModule(builder, pModule, propertyName);
                break;
        }
    });
    return storeOptions;
}
function getState(pState, pNamespace) {
    var state = pState;
    for (var _i = 0, pNamespace_1 = pNamespace; _i < pNamespace_1.length; _i++) {
        var namespace = pNamespace_1[_i];
        state = state[namespace];
    }
    return state;
}
function bindState(pBuilder, pModule, propertyName) {
    var value = pModule[propertyName];
    var provider = pBuilder.provider;
    var options = pBuilder.options;
    Object.defineProperty(pModule, propertyName, {
        get: function () {
            var state = provider.store
                ? getState(provider.store.state, pBuilder.namespaces)
                : options.state;
            return state[propertyName];
        },
        set: function (val) {
            var state = provider.store
                ? getState(provider.store.state, pBuilder.namespaces)
                : options.state;
            state[propertyName] = val;
        }
    });
    pModule[propertyName] = value;
}
function getGetter(pModule, propertyName) {
    var getter = pModule.__lookupGetter__(propertyName);
    if (!getter) {
        var className = pModule.constructor.name;
        throw {
            code: 'getter_not_found',
            message: "Could not find getter \"" + propertyName + "\" for module " + className,
            params: {
                className: className,
                propertyName: propertyName
            }
        };
    }
    var boundGetter = getter.bind(pModule);
    return function (state) { return boundGetter(); };
}
function bindGetter(pBuilder, pModule, propertyName) {
    var options = pBuilder.options;
    var provider = pBuilder.provider;
    options.getters[propertyName] = getGetter(pModule, propertyName);
    var getterName = pBuilder.namespaces.join('/') + '/' + propertyName;
    Object.defineProperty(pModule, propertyName, {
        get: function () {
            return provider.store.getters[getterName];
        }
    });
}
function getMutation(pModule, propertyName) {
    var mutation = pModule[propertyName];
    var boundMutation = mutation.bind(pModule);
    return function (state, payload) { return boundMutation(payload); };
}
function bindMutation(pBuilder, pModule, propertyName) {
    var options = pBuilder.options;
    var provider = pBuilder.provider;
    options.mutations[propertyName] = getMutation(pModule, propertyName);
    var mutationName = pBuilder.namespaces.join('/') + '/' + propertyName;
    pModule[propertyName] = function (payload) {
        provider.store.commit(mutationName, payload);
    };
}
function getAction(pModule, propertyName) {
    var action = pModule[propertyName];
    var boundAction = action.bind(pModule);
    return function (state, payload) { return boundAction(payload); };
}
function bindAction(pBuilder, pModule, propertyName) {
    var options = pBuilder.options;
    var provider = pBuilder.provider;
    options.actions[propertyName] = getAction(pModule, propertyName);
    var actionName = pBuilder.namespaces.join('/') + '/' + propertyName;
    pModule[propertyName] = function (payload) {
        provider.store.dispatch(actionName, payload);
    };
}
function bindModule(pBuilder, pModule, propertyName) {
    var subModule = pModule[propertyName];
    if (!subModule) {
        return;
    }
    var moduleOptions = transformRecursive(pBuilder.provider, subModule, pBuilder.namespaces.concat(propertyName));
    var options = pBuilder.options;
    options.modules[propertyName] = moduleOptions;
    Object.defineProperty(pModule, propertyName, {
        get: function () {
            return subModule;
        }
    });
}
//# sourceMappingURL=store.js.map